// Copyright (c) 2021, Philipp Matthias Schäfer (philipp.matthias.schaefer@posteo.de)
//
// This file is part of the WebLDAPPasswd application.
//
// The WebLDAPPasswd application is free software: you can redistribute it
// and/or modify it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the License or
// (at your option) any later version.
//
// The WebLDAPPasswd application is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
// General Public License for more details.
//
// You should have received a copy of the GNU General Affero Public License
// along with the WebLDAPPasswd. If not, see <https://www.gnu.org/licenses/>.
use handlebars::Handlebars;
use ldap3::LdapConn;
use ldap3::exop::PasswordModify;
use ldap3::result::{LdapError, Result};
use rocket_contrib::json::Json;

use rocket::post;
use serde_derive::{Deserialize, Serialize};
use serde_json::json;

use crate::context::Context;

#[derive(Deserialize)]
pub struct PasswordData {
    username: String,
    old_password: String,
    new_password: String,
}

#[derive(Serialize)]
enum Message {
    InvalidCredentials,
    ServerError,
}

#[derive(Serialize)]
pub struct Response {
    success: bool,
    message: Option<Message>,
}

fn change_password(data: &Json<PasswordData>,
                   context: &rocket::State<Context>) -> Result<()> {

    let dn = Handlebars::new()
        .render_template(&context.dn, &json!({"username" : &data.username}))
        .expect("Unexpected DN template error. Was tested in config.rs");

    let mut ldap = LdapConn::new(&context.ldap_url)?;
    ldap
        .simple_bind(&dn, &data.old_password)?
        .success()?;

    ldap
        .extended(PasswordModify{
            user_id: Some(&dn),
            old_pass: Some(&data.old_password),
            new_pass: Some(&data.new_password),
        })?
        .success()?;

    Ok(())
}

#[post("/update", data = "<data>")]
pub fn update(data: Json<PasswordData>,
              context: rocket::State<Context>) -> Json<Response> {
    Json(
        match change_password(&data, &context) {
            Ok(_) => Response{
                success: true,
                message: None,
            },
            Err(error) => {
                eprintln!("LDAP error: {}", error);
                Response {
                    success: false,
                    message: Some(
                        match error {
                            LdapError::LdapResult{ result } => {
                                if result.rc == 49 {
                                    Message::InvalidCredentials
                                } else {
                                    Message::ServerError
                                }
                            },
                            _ => Message::ServerError,
                        }
                    ),
                }
            },
        }
    )
}
